import babel from 'rollup-plugin-babel';
import svelte from 'rollup-plugin-svelte';
import resolve from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import livereload from 'rollup-plugin-livereload';
import { terser } from 'rollup-plugin-terser';
import builtins from 'rollup-plugin-node-builtins';
import globals from 'rollup-plugin-node-globals';
import sveltePreprocess from 'svelte-preprocess';
import typescript from '@rollup/plugin-typescript';
import json from "@rollup/plugin-json";
import scss from 'rollup-plugin-scss'
import { nodeResolve } from '@rollup/plugin-node-resolve';

const production = !process.env.ROLLUP_WATCH;
const port = process.env.PORT;
const buildDir = 'wwwroot/build';

function serve() {
	let server;
	
	function toExit() {
		if (server) server.kill(0);
	}

	return {
		writeBundle() {
			if (server) return;
			server = require('child_process').spawn('npm', ['run', 'start', '--', '--dev'], {
				stdio: ['ignore', 'inherit', 'inherit'],
				shell: true
			});

			process.on('SIGTERM', toExit);
			process.on('exit', toExit);
		}
	};
}

export default {
	input: 'src/main.ts',
	output: {
		sourcemap: true,
		format: 'esm',
		name: 'app',
		//file: 'wwwroot/bundle.js'
		dir: buildDir,
	},
	plugins: [
		svelte({
			// enable run-time checks when not in production
			dev: !production,
			// // we'll extract any component CSS out into
			// // a separate file - better for performance
			// css: css => {
			// 	css.write('bundle.css');
			// },
			preprocess: sveltePreprocess({
				scss: {
                     includePaths: ['src'],
                     excludePaths: ['wwwroot', 'node_modules'],
					minify: !production
                 },
                 postcss: {
                     plugins: [require('autoprefixer')],
                 },
			}),
			emitCss: true
		}),

		// If you have external dependencies installed from
		// npm, you'll most likely need these plugins. In
		// some cases you'll need additional configuration -
		// consult the documentation for details:
		// https://github.com/rollup/plugins/tree/master/packages/commonjs
		resolve({
			browser: true,
			dedupe: importee => importee === 'svelte' || importee.startsWith('svelte/')
		}),
		commonjs({
			extensions: ['.js'],
			namedExports: {
				'node_modules/bootstrap/dist/js/bootstrap.min.js' : ['bootstrap'],
				jquery : 'jQuery', 
				'popper.js': 'Popper'
			},
			exclude: "node_modules/**"
		}),
		json(),
        globals(),
        builtins(),
        babel({exclude: "node_modules/**"}),
		nodeResolve(),
		typescript({ sourceMap: !production }),
		scss({
			exclude: ['node_modules'],
			sass: require('sass'),
			minify: !production,
			output: 'wwwroot/build/bundle.css',
			sourcemap: true
		}),

		// In dev mode, call `npm run start` once
		// the bundle has been generated
		//!production && serve(),

		// Watch the `wwwroot` directory and refresh the
		// browser on changes when not in production
		// note that livereload will not work when running under aspnetcore. use the livereload nuget package when running under aspnetcore
		!production && livereload({
                watch: './wwwroot',
                port,
            }),

		// If we're building for production (npm run build
		// instead of npm run dev), minify
		production && terser()
	],
	watch: {
		clearScreen: false
	}
};
